"""J.A.R.V.I.S. Complete the Code 2 Cog."""
import re
from datetime import datetime, timedelta

import aiohttp
from ButtonPaginator import Paginator
from discord import Member, User
from discord.ext import commands
from discord_slash import SlashContext, cog_ext
from discord_slash.model import ButtonStyle

from jarvis.db.models import Guess
from jarvis.utils import build_embed
from jarvis.utils.cachecog import CacheCog
from jarvis.utils.field import Field

guild_ids = [578757004059738142, 520021794380447745, 862402786116763668]

valid = re.compile(r"[\w\s\-\\/.!@#$%^*()+=<>,\u0080-\U000E0FFF]*")
invites = re.compile(
    r"(?:https?://)?(?:www.)?(?:discord.(?:gg|io|me|li)|discord(?:app)?.com/invite)/([^\s/]+?)(?=\b)",
    flags=re.IGNORECASE,
)


class CTCCog(CacheCog):
    """J.A.R.V.I.S. Complete the Code 2 Cog."""

    def __init__(self, bot: commands.Bot):
        super().__init__(bot)
        self._session = aiohttp.ClientSession()
        self.url = "https://completethecodetwo.cards/pw"

    def __del__(self):
        self._session.close()

    @cog_ext.cog_subcommand(
        base="ctc2",
        name="about",
        description="CTC2 related commands",
        guild_ids=guild_ids,
    )
    @commands.cooldown(1, 30, commands.BucketType.channel)
    async def _about(self, ctx: SlashContext) -> None:
        await ctx.send("See https://completethecode.com for more information")

    @cog_ext.cog_subcommand(
        base="ctc2",
        name="pw",
        description="Guess a password for https://completethecodetwo.cards",
        guild_ids=guild_ids,
    )
    @commands.cooldown(1, 2, commands.BucketType.user)
    async def _pw(self, ctx: SlashContext, guess: str) -> None:
        if len(guess) > 800:
            await ctx.send(
                "Listen here, dipshit. Don't be like <@256110768724901889>. Make your guesses < 800 characters.",
                hidden=True,
            )
            return
        elif not valid.fullmatch(guess):
            await ctx.send(
                "Listen here, dipshit. Don't be like <@256110768724901889>. Make your guesses *readable*.",
                hidden=True,
            )
            return
        elif invites.search(guess):
            await ctx.send(
                "Listen here, dipshit. No using this to bypass sending invite links.",
                hidden=True,
            )
            return
        guessed = Guess.objects(guess=guess).first()
        if guessed:
            await ctx.send("Already guessed, dipshit.", hidden=True)
            return
        result = await self._session.post(self.url, data=guess)
        correct = False
        if 200 <= result.status < 400:
            await ctx.send(f"{ctx.author.mention} got it! Password is {guess}!")
            correct = True
        else:
            await ctx.send("Nope.", hidden=True)
        _ = Guess(guess=guess, user=ctx.author.id, correct=correct).save()

    @cog_ext.cog_subcommand(
        base="ctc2",
        name="guesses",
        description="Show guesses made for https://completethecodetwo.cards",
        guild_ids=guild_ids,
    )
    @commands.cooldown(1, 2, commands.BucketType.user)
    async def _guesses(self, ctx: SlashContext) -> None:
        exists = self.check_cache(ctx)
        if exists:
            await ctx.defer(hidden=True)
            await ctx.send(
                f"Please use existing interaction: {exists['paginator']._message.jump_url}",
                hidden=True,
            )
            return

        guesses = Guess.objects().order_by("-correct", "-id")
        fields = []
        for guess in guesses:
            user = ctx.guild.get_member(guess["user"])
            if not user:
                user = await self.bot.fetch_user(guess["user"])
                if not user:
                    user = "[redacted]"
            if isinstance(user, (Member, User)):
                user = user.name + "#" + user.discriminator
            name = "Correctly" if guess["correct"] else "Incorrectly"
            name += " guessed by: " + user
            fields.append(
                Field(
                    name=name,
                    value=guess["guess"] + "\n\u200b",
                    inline=False,
                )
            )
        pages = []
        for i in range(0, len(fields), 5):
            embed = build_embed(
                title="completethecodetwo.cards guesses",
                description=f"{len(fields)} guesses so far",
                fields=fields[i : i + 5],  # noqa: E203
                url="https://completethecodetwo.cards",
            )
            embed.set_thumbnail(url="https://dev.zevaryx.com/db_logo.png")
            embed.set_footer(
                text="dbrand.com",
                icon_url="https://dev.zevaryx.com/db_logo.png",
            )
            pages.append(embed)

        paginator = Paginator(
            bot=self.bot,
            ctx=ctx,
            embeds=pages,
            timeout=60 * 5,  # 5 minute timeout
            only=ctx.author,
            disable_after_timeout=True,
            use_extend=len(pages) > 2,
            left_button_style=ButtonStyle.grey,
            right_button_style=ButtonStyle.grey,
            basic_buttons=["◀", "▶"],
        )

        self.cache[hash(paginator)] = {
            "guild": ctx.guild.id,
            "user": ctx.author.id,
            "timeout": datetime.utcnow() + timedelta(minutes=5),
            "command": ctx.subcommand_name,
            "paginator": paginator,
        }

        await paginator.start()


def setup(bot: commands.Bot) -> None:
    """Add CTCCog to J.A.R.V.I.S."""
    bot.add_cog(CTCCog(bot))
