"""J.A.R.V.I.S. error handling cog."""
from discord.ext import commands
from discord_slash import SlashContext

from jarvis import slash


class ErrorHandlerCog(commands.Cog):
    """J.A.R.V.I.S. error handling cog."""

    def __init__(self, bot: commands.Bot):
        self.bot = bot

    @commands.Cog.listener()
    async def on_command_error(self, ctx: commands.Context, error: Exception) -> None:
        """d.py on_command_error override."""
        if isinstance(error, commands.errors.MissingPermissions):
            await ctx.send("I'm afraid I can't let you do that.")
        elif isinstance(error, commands.errors.CommandNotFound):
            return
        elif isinstance(error, commands.errors.CommandOnCooldown):
            await ctx.send(
                "Command on cooldown. " + f"Please wait {error.retry_after:0.2f}s before trying again",
            )
        else:
            await ctx.send(f"Error processing command:\n```{error}```")
            ctx.command.reset_cooldown(ctx)

    @commands.Cog.listener()
    async def on_slash_command_error(self, ctx: SlashContext, error: Exception) -> None:
        """discord_slash on_slash_command_error override."""
        if isinstance(error, commands.errors.MissingPermissions) or isinstance(error, commands.errors.CheckFailure):
            await ctx.send("I'm afraid I can't let you do that.", hidden=True)
        elif isinstance(error, commands.errors.CommandNotFound):
            return
        elif isinstance(error, commands.errors.CommandOnCooldown):
            await ctx.send(
                "Command on cooldown. " + f"Please wait {error.retry_after:0.2f}s before trying again",
                hidden=True,
            )
        else:
            await ctx.send(
                f"Error processing command:\n```{error}```",
                hidden=True,
            )
            raise error
            slash.commands[ctx.command].reset_cooldown(ctx)


def setup(bot: commands.Bot) -> None:
    """Add ErrorHandlerCog to J.A.R.V.I.S."""
    bot.add_cog(ErrorHandlerCog(bot))
