"""J.A.R.V.I.S. database object for mongoengine."""
from datetime import datetime

from mongoengine import Document
from mongoengine.fields import (
    BooleanField,
    DateTimeField,
    DictField,
    DynamicField,
    IntField,
    ListField,
    LongField,
    StringField,
)


class SnowflakeField(LongField):
    """Snowflake LongField Override."""

    pass


class Autopurge(Document):
    """Autopurge database object."""

    guild = SnowflakeField(required=True)
    channel = SnowflakeField(required=True)
    delay = IntField(min_value=1, max_value=300, default=30)
    admin = SnowflakeField(required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Autoreact(Document):
    """Autoreact database object."""

    guild = SnowflakeField(required=True)
    channel = SnowflakeField(required=True)
    reactions = ListField(field=StringField())
    admin = SnowflakeField(required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Ban(Document):
    """Ban database object."""

    active = BooleanField(default=True)
    admin = SnowflakeField(required=True)
    user = SnowflakeField(required=True)
    username = StringField(required=True)
    discrim = IntField(min_value=1, max_value=9999, required=True)
    duration = IntField(min_value=1, max_value=744, required=False)
    guild = SnowflakeField(required=True)
    type = StringField(default="perm", max_length=4, required=True)
    reason = StringField(max_length=100, required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Config(Document):
    """Config database object."""

    key = StringField(required=True)
    value = DynamicField(required=True)

    meta = {"db_alias": "main"}


class Guess(Document):
    """Guess database object."""

    correct = BooleanField(default=False)
    guess = StringField(max_length=800, required=True)
    user = SnowflakeField(required=True)

    meta = {"db_alias": "ctc2"}


class Joke(Document):
    """Joke database object."""

    rid = StringField()
    body = StringField()
    title = StringField()
    created_utc = DateTimeField()
    over_18 = BooleanField()
    score = IntField()

    meta = {"db_alias": "main"}


class Kick(Document):
    """Kick database object."""

    admin = SnowflakeField(required=True)
    guild = SnowflakeField(required=True)
    reason = StringField(max_length=100, required=True)
    user = SnowflakeField(required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Lock(Document):
    """Lock database object."""

    active = BooleanField(default=True)
    admin = SnowflakeField(required=True)
    channel = SnowflakeField(required=True)
    duration = IntField(min_value=1, max_value=300, default=10)
    guild = SnowflakeField(required=True)
    reason = StringField(max_length=100, required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Mute(Document):
    """Mute database object."""

    active = BooleanField(default=True)
    user = SnowflakeField(required=True)
    admin = SnowflakeField(required=True)
    duration = IntField(min_value=-1, max_value=300, default=10)
    guild = SnowflakeField(required=True)
    reason = StringField(max_length=100, required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Purge(Document):
    """Purge database object."""

    admin = SnowflakeField(required=True)
    channel = SnowflakeField(required=True)
    guild = SnowflakeField(required=True)
    count = IntField(min_value=1, default=10)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Reminder(Document):
    """Reminder database object."""

    active = BooleanField(default=True)
    user = SnowflakeField(required=True)
    guild = SnowflakeField(required=True)
    channel = SnowflakeField(required=True)
    message = StringField(max_length=100, required=True)
    remind_at = DateTimeField(required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Rolegiver(Document):
    """Rolegiver database object."""

    guild = SnowflakeField(required=True)
    roles = ListField(field=SnowflakeField())

    meta = {"db_alias": "main"}


class Roleping(Document):
    """Roleping database object."""

    active = BooleanField(default=True)
    role = SnowflakeField(required=True)
    guild = SnowflakeField(required=True)
    admin = SnowflakeField(required=True)
    bypass = DictField()
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Setting(Document):
    """Setting database object."""

    guild = SnowflakeField(required=True)
    setting = StringField(required=True)
    value = DynamicField()

    meta = {"db_alias": "main"}


class Star(Document):
    """Star database object."""

    active = BooleanField(default=True)
    index = IntField(required=True)
    message = SnowflakeField(required=True)
    channel = SnowflakeField(required=True)
    starboard = SnowflakeField(required=True)
    guild = SnowflakeField(required=True)
    admin = SnowflakeField(required=True)
    star = SnowflakeField(required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Starboard(Document):
    """Starboard database object."""

    channel = SnowflakeField(required=True)
    guild = SnowflakeField(required=True)
    admin = SnowflakeField(required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Twitter(Document):
    """Twitter Follow object."""

    active = BooleanField(default=True)
    handle = StringField(required=True)
    channel = SnowflakeField(required=True)
    guild = SnowflakeField(required=True)
    last_tweet = SnowflakeField(required=True)
    retweets = BooleanField(default=True)
    admin = SnowflakeField(required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Unban(Document):
    """Unban database object."""

    user = SnowflakeField(required=True)
    username = StringField(required=True)
    discrim = IntField(min_value=1, max_value=9999, required=True)
    guild = SnowflakeField(required=True)
    admin = SnowflakeField(required=True)
    reason = StringField(max_length=100, required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}


class Warning(Document):
    """Warning database object."""

    active = BooleanField(default=True)
    admin = SnowflakeField(required=True)
    user = SnowflakeField(required=True)
    guild = SnowflakeField(required=True)
    duration = IntField(min_value=1, max_value=120, default=24)
    reason = StringField(max_length=100, required=True)
    created_at = DateTimeField(default=datetime.utcnow)

    meta = {"db_alias": "main"}
