"""Cog wrapper for command caching."""
from datetime import datetime, timedelta

from discord.ext import commands
from discord.ext.tasks import loop
from discord.utils import find
from discord_slash import SlashContext


class CacheCog(commands.Cog):
    """Cog wrapper for command caching."""

    def __init__(self, bot: commands.Bot):
        self.bot = bot
        self.cache = {}
        self._expire_interaction.start()

    def check_cache(self, ctx: SlashContext, **kwargs: dict) -> dict:
        """Check the cache."""
        if not kwargs:
            kwargs = {}
        return find(
            lambda x: x["command"] == ctx.subcommand_name  # noqa: W503
            and x["user"] == ctx.author.id  # noqa: W503
            and x["guild"] == ctx.guild.id  # noqa: W503
            and all(x[k] == v for k, v in kwargs.items()),  # noqa: W503
            self.cache.values(),
        )

    @loop(minutes=1)
    async def _expire_interaction(self) -> None:
        keys = list(self.cache.keys())
        for key in keys:
            if self.cache[key]["timeout"] <= datetime.utcnow() + timedelta(minutes=1):
                del self.cache[key]
