"""Permissions wrappers."""
from discord.ext import commands

from jarvis.config import get_config


def user_is_bot_admin() -> bool:
    """Check if a user is a J.A.R.V.I.S. admin."""

    def predicate(ctx: commands.Context) -> bool:
        """Command check predicate."""
        if getattr(get_config(), "admins", None):
            return ctx.author.id in get_config().admins
        else:
            return False

    return commands.check(predicate)


def admin_or_permissions(**perms: dict) -> bool:
    """Check if a user is an admin or has other perms."""
    original = commands.has_permissions(**perms).predicate

    async def extended_check(ctx: commands.Context) -> bool:
        """Extended check predicate."""  # noqa: D401
        return await commands.has_permissions(administrator=True).predicate(ctx) or await original(ctx)

    return commands.check(extended_check)
